/*
   Test program for PCI-12/14bit A/D,D/A,D/I,DO
*/

#include <dos.h>
#include <stdio.h>
#include <conio.h>
#include <stdlib.h>

#include "pci.h"

#define ADC_CHANNEL	0x000F
#define ADC_SCALE	0x0070
#define ADC_SAMPLE	0x0080
#define ADC_TRIG_INT	0x0000
#define ADC_TRIG_EXT	0x0200
#define ADC_TRIG_8254	0x0300
#define ADC_EXT_GATE	0x0400
#define ADC_INTR	0x0800
#define ADC_TRIG	0x1000
#define ADC_HOLD	(~ADC_SAMPLE)

/* set default value for ioport, CNTR */
unsigned int ioport = 0;
unsigned int picbase = 0x20;
unsigned int picmask = 0;
unsigned long intcount = 0L;
unsigned int intdupcount = 0;
unsigned int irq = 0;
unsigned int adbit = 12;
unsigned int ssid = 0;

int old_adr = 0;


#define NUM_UNSIGNED     1
#define NUM_1_COMPLEMENT 2
#define NUM_2_COMPLEMENT 3
#define NUM_EXCEED       4

float num_volt = 10.0;
int num_system = NUM_2_COMPLEMENT;

static void interrupt (*oldisr) (void) = NULL;

static void interrupt isr (void)
{
//  disable ();
  intcount++;
  inport (ioport + 16);

  if (++intdupcount >= 50)
    outportb (picbase + 1, inportb (picbase + 1) | picmask);
  (*oldisr) ();
//  enable ();
  outportb (picbase, 0x20);
}

void Set_IRQ (void)
{
  int vect;

  if (0 == irq)
    return;
  picbase = (irq < 8)? 0x20 : 0xA0;
  picmask = 1 << (irq & 7);
  vect = (irq < 8)? 8 + irq : 0x68 + irq;
  oldisr = getvect (vect);
  setvect (vect, isr);
  outportb (picbase + 1, inportb (picbase + 1) & ~picmask);
}

void Reset_IRQ (void)
{
  int vect;

  if (0 == irq)
    return;
  outportb (picbase + 1, inportb (picbase + 1) | picmask);
  vect = (irq < 8)? 8 + irq : 0x68 + irq;
  setvect (vect, oldisr);
}

void Set_DA (int ch, int val)
{
  int OFS = 2;

  if (old_adr) OFS <<= 3;
  if (ch != 1 && ch != 2)
    return;
  outport (ioport + OFS * ch, val);
}

void Set_DO (unsigned x)
{
  int OFS = 6;

  if (old_adr) OFS <<= 3;
  outport (ioport + OFS, x);
}

unsigned int Get_DI (void)
{
  int OFS = 6;

  if (old_adr) OFS <<= 3;
  outport (ioport + OFS, 0xffff);
  return inport (ioport + OFS);
}

void Set_2VoltageRule (float range, int numsys)
{
  num_volt = range;
  num_system = numsys;
}

float Get_Voltage (int x)
{
  float voltage;
  int admax = (1 << adbit) - 1;

  voltage = ((float) num_volt / (admax + 1)) * x;

  return voltage;
}

void Test_AD_Loopback (void)
{
  unsigned int adc, nGain = 0, nChannel;
  int i, x, da, count, err, c;
  long int bias[16], mean[16], max[16];
  int admax = (1 << adbit) - 1;
  int OFS = 2;

  if (old_adr) OFS <<= 3;
  clrscr ();
  gotoxy (30, 2);
  cprintf ("<AD/DA loop-back test>");
  gotoxy (8, 4);
  cprintf ("Channel  Input  Error   MeanErr      Bias      MaxErr");
  for (i = 0; i < 16; i++) {
    bias[i] = mean[i] = max[i] = 0;
  }

  outport (ioport + OFS, 0);
  adc = ADC_TRIG_INT | ADC_INTR | ADC_SAMPLE;
  outport (ioport, adc);
  delay (1);
  adc &= ADC_HOLD;
  outport (ioport, adc);
  adc |= ADC_TRIG;
  outport (ioport, adc);
  adc &= ~ADC_TRIG;
  outport (ioport, adc);

  delay (100);
  count = -2;
  for (da = admax; da >= 0; da -= 5) {
    count++;
    outport (ioport + OFS, da);
    gotoxy (3, 22);
    cprintf ("D/A output: %6d", da);
    delay (10);
    for (nChannel = 0; nChannel < 16; nChannel++) {
      adc = nChannel | (nGain << 4) | ADC_TRIG_INT | ADC_INTR | ADC_SAMPLE;
      outport (ioport, adc);
      delay (1);
      adc &= ADC_HOLD;
      outport (ioport, adc);
      adc |= ADC_TRIG;
      outport (ioport, adc);
      adc &= ~ADC_TRIG;
      outport (ioport, adc);
      x = inport (ioport);
      x &= admax;
      if (count <= 0)
	continue;
      bias[nChannel] += x - da;
      err = abs (x - da);
      if (max[nChannel] < err)
	max[nChannel] = err;
      mean[nChannel] += err;
      gotoxy (10, 5 + nChannel);
      cprintf ("%2d :%7d %5d     %7.4f    %7.4f  %7ld", nChannel + 1, x, err,
      (float)mean[nChannel] / count,
      (float)bias[nChannel] / count,
      max[nChannel]);
    }
    if (kbhit ()) {
      c = getch ();
      if (c == 27)
	break;
      else if (c == ' ')
	getch ();
    }
  }
  getch ();
}

int Get_AD (int nChannel, int nGain)
{
  int admax = (1 << adbit) - 1;
  int signbit = 1 << (adbit - 1);
  unsigned int adc = 0;
  unsigned int x = 0;
  int OFS = 0;

  if (old_adr) OFS <<= 3;
  if (nChannel < 0 || nChannel > 15)
    return 0;
  if (nGain < 0 || nGain > 7)
    return 0;
  adc = nChannel | (nGain << 4) | ADC_TRIG_INT | ADC_INTR | ADC_SAMPLE;
//  adc = nChannel | (nGain << 4) | ADC_TRIG_INT | ADC_SAMPLE;
  outport (ioport + OFS, adc);
//  delay (5);
  delay (1);
  adc &= ADC_HOLD;
  outport (ioport + OFS, adc);
  adc |= ADC_TRIG;
  outport (ioport + OFS, adc);
  adc &= ~ADC_TRIG;
  outport (ioport + OFS, adc);
  x = inport (ioport + OFS);
  x &= admax;
  switch (num_system) {
    case NUM_UNSIGNED:
      break;
    case NUM_2_COMPLEMENT:
      if (x & signbit)
	x = x - (1 << adbit);
      break;
    case NUM_1_COMPLEMENT:
      if (x & signbit)
	x = signbit - x;
      break;
    case NUM_EXCEED:
	x = x - signbit;
      break;
  }
  return x;
}

void Show_AD_Value (int nChannels, int nGain)
{
  int x, n;
  float voltage;

  for (n = 0;;) {
    x = Get_AD (n, nGain);
    gotoxy (3, 2 + n);
    voltage = Get_Voltage (x);
    gotoxy (3, 2 + n);
    cprintf ("Channel#%2d :%7d      %8.4f V", n, x, voltage);
    if (n < nChannels - 1)
      n++;
    else
      n = 0;
    if (kbhit ())
      break;
  }
}

void Calibrate_AD12 (void)
{
  int admax = (1 << adbit) - 1;
  int vrange = 10;
  int numsys = NUM_2_COMPLEMENT;

  Set_2VoltageRule (vrange, numsys);

  clrscr ();
  gotoxy (3, 19);
  cprintf ("JP2  JP3  JP4  JP5  (Open JP1 if AD526 is used)");
  gotoxy (3, 20);
  cprintf (" 23   23  12   12   Adjust VR3 and VR4 to %d\n", admax / 2);
  Show_AD_Value (16, 0);
  while (kbhit()) getch ();
  gotoxy (3, 21);
  cprintf (" 23   23  12    23   Adjust VR2 to 0\n");
  Show_AD_Value (16, 0);
  while (kbhit()) getch ();
  gotoxy (3, 22);
  cprintf (" 23  12   12   12    Adjust VR1 to %d\n", admax / 2);
  Show_AD_Value (16, 0);
  while (kbhit()) getch ();
}

/*
void testadinput (int adbit)
{
  unsigned int adc, x, nGain = 0, nChannel;
  long loop;
  int c, i, err, duration = 8, step = 0;
  int admax = (1 << adbit) - 1;
  int da = admax / 2;
  int OFS = 64 + 12;

  if (old_adr) OFS <<= 3;

  clrscr ();
  gotoxy (3, 24);
  cprintf ("1 : scale   2, 3 : Trigger Source");

  outport (ioport + 64 + 12, 0x36); //Counter 0: Mode 3, LSB + MSB
  outport (ioport + 64, 100);
  outport (ioport + 64, 0);

  outportb (ioport + 64 + 12, 0x76); //Counter 1: Mode 3, LSB + MSB
  outportb (ioport + 64 + 4, 100);
  outportb (ioport + 64 + 4, 0);

  inport (ioport + 16);// clear irq
  setirq ();

  for (loop = 0;;loop++) {
    outport (ioport + 16, da);
    adc = nChannel | (nGain << 4) | ADC_TRIG_8254 | ADC_INTR;
    //adc = nChannel | (nGain << 4) | ADC_TRIG_INT | ADC_INTR | ADC_SAMPLE;
    outport (ioport, adc);
    for (nChannel = 0; nChannel < 16; nChannel ++) {
      intdupcount = 0;
      adc = nChannel | (nGain << 4) | ADC_TRIG_8254 | ADC_INTR;
      //adc = nChannel | (nGain << 4) | ADC_TRIG_INT | ADC_INTR;
      outport (ioport, adc);
      //delay (duration);
      //adc &= ADC_HOLD;
      //outport (ioport, adc);
      //delay (1);
      //adc |= ADC_TRIG;
      //outport (ioport, adc);
      //adc &= ~ADC_TRIG;
      //outport (ioport, adc);
      for (i = 0; i < 5; i++) {
	if (intdupcount > 0)
	  break;
	delay (1);
      }
      x = inport (ioport);
      err = abs (x - da);
      gotoxy (3, 5 + nChannel);
      cprintf ("Channel#%2d :%7d  %5d  %5d", nChannel, x, err, intdupcount);
    }
    gotoxy (3, 22);
    cprintf ("Loop :%7ld   IRQ :%7ld   Scale = %2d   D/A : %4d", loop, intcount, nGain + 1, da);
    while (kbhit () || step) {
      c = getch ();
      if (c == 27)
	break;
      else if (c == ' ')
	step = !step;
      else if (c == '+') {
	if (duration < 256)
	  duration <<= 1;
      }
      else if (c == '-') {
	if (duration > 1)
	  duration >>= 1;
      }
      else if (c == '1') {
	nGain++;
	if (nGain >= 8)
	  nGain = 0;
      }
      else if (c == '3') {
	da += 100;
	if (da > admax)
	  da = admax;
      }
      else if (c == '2') {
	da -= 100;
	if (da < 0)
	  da = 0;
      }
      else if (!step)
	c = getch ();
    }
    if (c == 27)
      break;
  } // for (loop)
  if (kbhit ())
    getch ();
  resetirq ();
}
*/

void Test_AD (void)
{
  //long loop;
  int c, x;
  int numsys = NUM_2_COMPLEMENT;
  int vrange = 10;

  if (adbit == 14) {
    switch (ssid) {
      case PCI_SID_AD7899AR1:
	numsys = NUM_2_COMPLEMENT;
	vrange = 20;
	break;

      case PCI_SID_AD7899AR2:
	numsys = NUM_UNSIGNED;
	vrange = 10;
	break;

      case PCI_SID_AD7899AR3:
	numsys = NUM_2_COMPLEMENT;
	vrange = 5;
	break;

    }
    Set_2VoltageRule (vrange, numsys);
  }

  //loop = 0;
  x = 0;
  clrscr ();
  for (;;) {
    gotoxy (3, 23);
    cprintf ("Scale: %3d", 1 << x);
    Show_AD_Value (16, x);
    while (kbhit ()) {
      c = getch ();
      switch (c) {
	case '1':
	  if (x > 0)
	    x--;
	  break;
	case '2':
	  if (x < 4)
	    x++;
	  break;
      }
      if (c == 27)
	break;
    }
    if (c == 27)
      break;
  }
}

void Calibrate_DA (int nChannel)
{
  int x = 0;
  unsigned char c = 0;
  int admax = (1 << adbit) - 1;

  gotoxy (3, 24);
  cprintf ("1: Min   2: Middle    3: Max    ESC: Continue");

  for (;;) {
    Set_DA (nChannel, x);
    gotoxy (3, 18);
    cprintf ("D/A Channel#%d :%7d", nChannel, x);
    if (kbhit ()) {
      c = getch ();
      if (c == 27)
	break;
      else if (c == '1') {
	x = 0;
      }
      else if (c == '2') {
	x = admax / 2 + 1;
      }
      else if (c == '3') {
	x = admax;
      }
    }
  } // for
  if (kbhit ())
    getch ();
}

void DA_Calibration (void)
{
  clrscr ();

  if (adbit == 12) {
    gotoxy (3, 10);
    cprintf ("1: JP5->12, Adjust VR5, -10 to 10 V");
    Calibrate_DA (1);
    gotoxy (3, 11);
    cprintf ("2: JP5->34, Adjust VR5, 0 to 10 V");
    Calibrate_DA (1);
    gotoxy (3, 12);
    cprintf ("3: JP6->12, Adjust VR6, -10 to 10 V");
    Calibrate_DA (2);
    gotoxy (3, 13);
    cprintf ("4: JP6->34, Adjust VR6, 0 to 10 V");
    Calibrate_DA (2);
  }
  else {
    gotoxy (3, 10);
    cprintf ("Channel 1: JP4->12 for Unipolar or JP4-23 for Bipolar");
    gotoxy (3, 11);
    cprintf ("Adjust VR1 for offset and VR3 for range");
    gotoxy (3, 12);
    cprintf ("Output: 0 - 10V for Unipolar, -10V to 10V for Bipolar");
    Calibrate_DA (1);
    gotoxy (3, 14);
    cprintf ("Channel 2: JP5->12 for Unipolar or JP5-23 for Bipolar");
    gotoxy (3, 15);
    cprintf ("Adjust VR2 for offset and VR4 for range");
    gotoxy (3, 16);
    cprintf ("Output: 0 - 10V for Unipolar, -10V to 10V for Bipolar");
    Calibrate_DA (2);
  }
}

void Test_DA (void)
{
  int chwave[2] = {3, 3};
  char *desc[8] = {"min",
		   "max",
		   "square",
		   "triangle",
		   "rising triangle",
		   "falling triangle",
		   "bit increasing",
		   "DC"};

  int x[2] = {0, 0};
  int y[2] = {0, 0};
  int dflag[2] = {0, 0};
  unsigned long loop = 0L;
  int i, flag, quit;
  int step = 0;
  unsigned char c = 0;
  unsigned duration = 27;
  int admax = (1 << adbit) - 1;
  int actch = 0;
  int dc[2] = {0, 0};

  clrscr ();
  gotoxy (30, 2);
  cprintf ("< DA test >");
  gotoxy (3, 20);
  cprintf ("Press 1 or 2 to change wave type.");
  gotoxy (3, 21);
  cprintf ("Press + - to change frequency.");
  gotoxy (3, 22);
  cprintf ("Press Up Down PgUp PgDn Home End to change DC value.");
  for (loop = 0, quit = 0;!quit;loop++) {
    Set_DA (1, x[0]);
    Set_DA (2, x[1]);
    gotoxy (10, 10);
    cprintf ("Active Channel: %d                DC Voltage   DC Voltage  DC Voltage", actch + 1);
    gotoxy (10, 11);
    cprintf ("Channel   Wave form             single polar  bi-polar(+) bi-polar(-)");
    gotoxy (10, 8);
    cprintf ("Loop:%9ld     D/A #1 :%7d     D/A #2 :%7d", loop, x[0], x[1]);
    for (i = 0; i < 2; i++) {
      gotoxy (10, 12 + i);
      cprintf ("   %d      %-20s %8.3fV      %8.3fV   %8.3fV",
	i + 1, desc[chwave[i]], (10.0 * dc[i]) / (admax + 1),
	(20.0 * dc[i]) / (admax + 1) - 10.0,
	10.0 - (20.0 * dc[i]) / (admax + 1));
      switch (chwave[i]) {
	case 0:
	  x[i] = 0;
	  break;
	case 1:
	  x[i] = admax;
	  break;
	case 2:
	  y[i] += duration;
	  if (y[i] > 1000) {
	    y[i] = 0;
	    x[i] = (x[i])? 0 : admax;
	  }
	  break;
	case 3:
	  if (dflag[i]) {
	    x[i] += duration;
	    if (x[i] > admax) {
	      x[i] = admax;
	      dflag[i] = 0;
	    }
	  } else {
	    x[i] -= duration;
	    if (x[i] < 0) {
	      x[i] = 0;
	      dflag[i] = 1;
	    }
	  }
	  break;
	case 4:
	  x[i] += duration;
	  if (x[i] > admax)
	    x[i] = 0;
	  break;
	case 5:
	  x[i] -= duration;
	  if (x[i] < 0)
	    x[i] = admax;
	  break;
	case 6:
	  x[i] <<= 1;
	  if (x[i] > admax)
	    x[i] = 1;
	  break;
	case 7:
	  x[i] = dc[i];
	  break;
      } //switch
    } //for

    if (kbhit () || step) {
      c = getch ();
      if (c == 0 && kbhit()) {
	c = getch ();
	switch (c) {
	  case 73:
	   dc[actch] += 1;
	   if (dc[actch] > admax)
	     dc[actch] = admax;
	   break;
	  case 72:
	   dc[actch] += 32;
	   if (dc[actch] > admax)
	     dc[actch] = admax;
	   break;
	  case 71:
	   dc[actch] += 1024;
	   if (dc[actch] > admax)
	     dc[actch] = admax;
	   break;
	  case 81:
	   dc[actch] -= 1;
	   if (dc[actch] < 0)
	     dc[actch] = 0;
	   break;
	  case 80:
	   dc[actch] -= 32;
	   if (dc[actch] < 0)
	     dc[actch] = 0;
	   break;
	  case 79:
	   dc[actch] -= 1024;
	   if (dc[actch] < 0)
	     dc[actch] = 0;
	   break;
	 case 75:
	   actch = 0;
	   break;
	 case 77:
	   actch = 1;
	   break;
	} //switch
      }
      else {
	switch (c) {
	  case 27:
	    quit = 1;
	    break;
	  case ' ':
	    step = !step;
	    break;
	  case '+':
	    if (duration < 200)
	      duration *= 3;
	    break;
	  case '-':
	    if (duration >= 3)
	      duration /= 3;
	    break;
	  case '1':
	    if (actch != 0) {
	      actch = 0;
	    } else {
	      chwave[0]++;
	      if (chwave[0] > 7)
		chwave[0] = 0;
	    }
	    break;
	  case '2':
	    if (actch != 1) {
	      actch = 1;
	    } else {
	      chwave[1]++;
	      if (chwave[1] > 7)
		chwave[1] = 0;
	    }
	    break;
	  default:
	    if (!step)
	      getch ();
	} // switch
      } //else
    } //if kbhit
  } // for (loop)
}

void Test_DIO_Loopback (void)
{
  int loop, i, c;
  unsigned x, y;

  int OFS = 6;

  if (old_adr) OFS <<= 3;
  for (loop = 0; loop < 10; loop ++) {
    for (i = 0; i < 16; i++) {
      x = 1 << i;
      outport (ioport + OFS, x);
      delay (10);
      y = inport (ioport + OFS);
      gotoxy (3, 21);
      cprintf("Loop: %3d  Testing Pin:%2d  Output: %04X  Input:%04X",
	       loop + 1, i, x, y);
      if (x != y) {
	cputs (" Error ");
	c = getch ();
	if (c == 27)
	  break;
      }
      else {
	cputs ("  OK   ");
	//c = getch ();
      }
      if (kbhit()) {
	c = getch ();
	c = getch ();
      }
      if (c == 27)
	break;
    }
    if (c == 27)
      break;
  }
  gotoxy (3, 17);
  cputs("Press any key when ready");
  if (kbhit ())
    getch();
}

void dumpbits (unsigned long x, int n)
{
  int i;
  unsigned long b = 1L << (n - 1);

  for (i = 0; i < n; i++) {
    putch (((b >> i) & x)? '1' : '0');
  }
}

void Test_DO (void)
{
  unsigned x;
  unsigned long loop = 0L;
  int step = 0;
  unsigned char c = 0;
  unsigned duration = 64;

  for (loop = 0, x = 1;;loop++) {
    if (x == 0x8000)
      x = 1;
    else
      x <<= 1;
    Set_DO (x);
    gotoxy (3, 21);
    cprintf("Loop: %8ld : ", loop);
    dumpbits (x, 16);
    delay (duration);
    if (kbhit () || step) {
      c = getch ();
      if (c == 27)
	break;
      else if (c == ' ')
	step = !step;
      else if (c == '+') {
	if (duration > 1)
	  duration >>= 1;
      }
      else if (c == '-') {
	if (duration < 512)
	  duration <<= 1;
      }
      else if (!step)
	getch ();
    }
  } // for (i)
  if (kbhit ())
    getch ();
}

void Test_DI (void)
{
  unsigned x;
  unsigned long loop = 0L;
  unsigned duration = 64;
  int step = 0;
  unsigned char c;

  for (loop = 0;;loop++) {
    x = Get_DI ();
    gotoxy (3, 21);
    cprintf("Loop: %8ld : ", loop);
    dumpbits (x, 16);
    delay (duration);
    if (kbhit () || step) {
      c = getch ();
      if (c == 27)
	break;
      else if (c == ' ')
	step = !step;
      else if (c == '+') {
	if (duration > 1)
	  duration >>= 1;
      }
      else if (c == '-') {
	if (duration < 512)
	  duration <<= 1;
      }
      else if (!step)
	getch ();
    }
  } // for (loop)
  if (kbhit ())
    getch ();
}

char *ssid2str (int ssid)
{
  switch (ssid) {
    case PCI_SID_AD7899AR1: return "AD7899AR-1";
    case PCI_SID_AD7899BR1: return "AD7899BR-1";
    case PCI_SID_AD7899SR1: return "AD7899SR-1";
    case PCI_SID_AD7899AR2: return "AD7899AR-2";
    case PCI_SID_AD7899AR3: return "AD7899AR-3";
    case PCI_SID_AD7899BR3: return "AD7899BR-3";
    case PCI_SID_AD7899ARS1: return "AD7899ARS-1";
    case PCI_SID_AD7899ARS2: return "AD7899ARS-2";
    case PCI_SID_AD7899ARS3: return "AD7899ARS-3";
    default: return "unknown";
  }
}

void main()
{
  int r;
  int flag = 1;
  char ch;

  while (flag) {
    clrscr();
    adbit = 12;
    r =  GetPCICardInfo (PCI_DEVICE_ID_12ADDA, &ioport, &irq);
    if (r)
      ioport = 0;
    if (ioport == 0) {
      adbit = 14;
      r =  GetPCICardInfo (PCI_DEVICE_ID_14ADDA, &ioport, &irq);
      if (r)
	ioport = 0;
      else
	GetPCISubsystemID (PCI_DEVICE_ID_14ADDA, &ssid);
    }
    if (ioport == 0) {
      cputs ("PCI-12/14 bit AD/DA Test Program: Card not found!\n\r");
      break;
    }
    cputs ("\n\n\n\r");
    cputs ("                PCI-12/14 bit AD/DA CARD Test Program V1.1\n\r");
    cputs ("                ==========================================\n\r");
    cprintf ("                       PCI %d bit AD/DA card found.\n\r", adbit);
    cprintf ("                       I/O Address: %04X   IRQ: %d\n\r", ioport, irq);
    if (adbit == 14)
      cprintf ("                       A/D: %s\n\r", ssid2str(ssid));
    cputs ("\n\r");
    cputs ("                       (1): ADA Loopback test\n\r");
    cputs ("                       (2): D/A Output test\n\r");
    cputs ("                       (3): A/D Input test\n\r");
    cputs ("                       (4): DIO Loopback test\n\r");
    cputs ("                       (5): DIO Output test\n\r");
    cputs ("                       (6): DIO Input test\n\r");
    cputs ("                       (7): D/A Calibration\n\r");
    if (adbit != 14)
      cputs ("                       (8): A/D Calibration\n\r");
    cputs ("                       ESC: EXIT\n\r");
    cputs ("                       Choose:");
    do {
      ch = getch();
    } while (ch != '1' && ch != '2' && ch != '3'
	  && ch != '4' && ch != '5' && ch != '6'
	  && ch != '7' && ch != '8' && ch != 27);

    switch (ch) {
      case 27:
	flag = 0;
	break;
      case '1':
	Test_AD_Loopback ();
	break;
      case '2':
	Test_DA ();
	break;
      case '3':
	Test_AD ();
	break;
      case '4':
	Test_DIO_Loopback ();
	break;
      case '5':
	Test_DO ();
	break;
      case '6':
	Test_DI ();
	break;
      case '7':
	DA_Calibration ();
	break;
      case '8':
	if (adbit != 14)
	  Calibrate_AD12 ();
	    break;
    }
  }
} /* end of main */
